
function  convertBasisE10(vE::Array{Int64,1}, E::Int64, N::Int64)
# converts the a vector vE into an integer p 
# vE is of length N, containing integers between 1 and E 
# p is the decimal represention of v (which is in basis E)
# convention: vE(1) represents the idiosyncratic  state in the present period and vE(N) represents the past idosyncratic state N-1 periods before

if (length(vE)!=N)
    println("incompatible length for vE");
    
end
if minimum(vE)<1
    println("all elements of vE should be >= 1");
    
end
if maximum(vE)>E
    println("all elements of vE should be <= E");
    
end
#v = vE - 1;
p = 0;
#for k=N:(-1):1
for k=1:N
    p = p+(vE[k]-1)*E^(N-k);
end

p=p+1;#otherwise between 0 and (E+1)^N - 1
    return p
end


function  convertBasis10E(p::Int64, E::Int64, N::Int64)
# converts the integer p into a vector vE of length N, containing integers between 1 and E+1 representing p in basis E
# convention: vE(1) represents the idiosyncratic  state in the present period and vE(N) represents the past idosyncratic state N-1 periods before

vE=Array{Int64,1}
    
if ((p<1)|(p>E^N))
    println("incompatible integer p");
    
end

ptemp = p-1;
vE = zeros(Int64,1,N);r=0;
for k=1:N 
    r = mod(ptemp, E);
    vE[N-k+1] = round(Int,1+r);
    ptemp = floor(Int,(ptemp-r)/(E));
end

    return vE
end


function Project_plan(       
        N::Integer, # length of the truncation
        AA0::AiyagariModelEGM,
        polA_ss::Array{T,1},
        polC_ss::Array{T,1},
        upc::Array{T,1},
        D_ss::Array{T,1},         
        Aggs_ss::AggVarsEGM{T,T},
        Mat::Array{T,2},
        A::T,
        B::T,
        vstan::Array{T,1},
        #Ls::Array{T,1},
       # Ulevel::Array{T,1},
        states::Array{T,1},        
        nc::Integer) where {T <: Real}

#polA_ss,polC_ss,D_ss,A,Aggs_ss,Mat,upc = steady(K0,pol0,AA0,R,w,PHI,τ,1e-4);    
    @unpack params,aGrid,na,dGrid,nd,ns,Transv,states = AA0
    @unpack γ,β,χ,ϕ = params
    
    R = Aggs_ss.R
    w = Aggs_ss.w
    Ltot = (χ*w)^ϕ


Trans   = reshape(Transv,ns,ns)



############################
# computing CC bins     
############################    
Vmat = Mat[1:na,:]
aGrid = AA0.aGrid
aGrid[1] = 0 
Va = repeat(aGrid,ns)'*Mat # policy rule in a a'(a)
pol = reshape(Va,na,ns)
Vind = zeros(Int,na*ns)
Wind = zeros(Float64,na*ns)
for i = 1:length(Va)
    indcc=findall(x->x<=Va[i],aGrid) 
    Vind[i] = maximum(indcc)
    Wind[i] = 1-(Va[i] - aGrid[Vind[i]])/(aGrid[Vind[i]+1]-aGrid[Vind[i]])
    
end
    

Ls  = (AA0.states*w*χ).^ϕ # vector of labor supplies
Lv = kron(Ls[1:ns],ones(na,1))

ytype = kron(collect(StepRange(1, Int8(1), ns)),ones(Int64,na)) # on the initial grid
Cpol =  - Va' + Aggs_ss.R*repeat(aGrid,ns) + Aggs_ss.w*states[ytype].*Lv + Aggs_ss.TT*ones(Float64,na*ns)  

Xe = ((χ*Aggs_ss.w .*states).^(1+ ϕ))/χ 
Xc = (ϕ/(1+ϕ))*Xe
Xc = kron(Xc,ones(na,1))  
    
resE = zeros(Float64,na*ns)
cimp = zeros(Float64,na*ns)
for i=1:na
    for j=1:ns        
            ind = i+(j-1)*na 
            EUc = 0
            for jp=1:ns                
                    EUc = EUc + Trans[jp,j] * (
                    (Wind[ind]*Cpol[Vind[ind]+(jp-1)*na] +
                    (1-Wind[ind])*Cpol[Vind[ind]+1+(jp-1)*na] -
                                Xc[Vind[ind]+(jp-1)*na])^-γ)
               
            end
            resE[ind] = (Cpol[ind] -Xc[ind] )^-γ - β*R*EUc
            cimp[ind] = (β*R*EUc)^(-1/γ)+Xc[ind]                
       
    end
end
 
 resi = upc - β*R*(Mat')*upc

#################    #################    #################    #################    
# I compute the vector of beginning-of-period  marginal utilities
#################    #################    #################    #################

#=    
Lv = [Aggs_ss.PHI*kron(Ls[1:ns],ones(na,1));(1-Aggs_ss.τ)*kron(Ls[1:ns],ones(na,1))]    
etype = [ones(Int,na*ns);2*ones(Int,na*ns)]
ytype = repeat(kron(collect(StepRange(1, Int8(1), ns)),ones(Int,na)),2)
Cpol =  - Va' + R*repeat(aGrid,ns*2) + w*states[ytype].*Lv  
    
    
Xe = ((χ*(1-τ)*w .*states).^(1+ ϕ))/χ 
Xc = (ϕ/(1+ϕ))*[Xe;Xe]    
Xc = kron(Xc,ones(na,1))  
    
for i=1:na
    for j=1:ns
        for k=1:2
            ind = i+(j-1)*na + (k-1)*na*ns
            upc[ind] = (Cpol[ind] -Xc[ind] )^-γ             
        end
    end
end
 =#   
    
# @show   AA = maximum(D_ss - Mat*D_ss)
      
    
    
    # @show   maximum(vstan - Mats*vstan)
    Nidio = ns
    Size = zeros(Float64,Nidio^N)
    compt=0
    Ntoi = zeros(Int,Nidio^N)
    for i=1:Nidio^N
     vE =  convertBasis10E(i, Nidio, N)
        s0 = vstan[vE[N]]
        Siz = s0
        for j=1:N-1
            Siz = Siz*Trans[vE[N-j],vE[N+1-j]]
        end
        if Siz>0
            Size[compt+1]=Siz
            Ntoi[compt+1]=i
            compt+=1
        end            
    end
    Nbin = compt
   
   
    
    
    
    
    CCv = zeros(Float64,na) # to compute the fraction of credit-constrained bin
    CCv[1] = 1
    CCv[2] = 1 # (for accuracy reason)
    CCv = repeat(CCv,Nidio)  # vector of 1 only for the lowest wealth bin for each Nidio
    
    
    dGridl_t = repeat(dGrid,ns)
    conso   = zeros(Float64,Nbin)
    at   = zeros(Float64,Nbin)
    ae   = zeros(Float64,Nbin)
    ae2   = zeros(Float64,Nbin)
    uc   = zeros(Float64,Nbin)
    Euc  = zeros(Float64,Nbin)
    ls   = zeros(Float64,Nbin) # labor supply of the bin
    Ul   = zeros(Float64,Nbin) # Utility level of the bin per capita
    xsiU = zeros(Float64,Nbin)
    xsiUc= zeros(Float64,Nbin)
    inc  = zeros(Float64,Nbin)
    resp = zeros(Float64,Nbin)
    CC = zeros(Float64,Nbin)
    X = zeros(Float64,Nbin) # Object within the utility function (c - xsi*l^..)
    Transu = spzeros(Nbin,Nbin); # general transition matrix AGENTS
    Size_check = zeros(Float64,Nbin)
    for k=1:Nbin
        i = Ntoi[k] #number of the history
        vE =  convertBasis10E(i, Nidio, N)
        yy = mod(vE[1] - 1 ,ns)+1 # productivity type between 1 and ns
        #ee = 1*(vE[1]<=ns)+2*(vE[1]>ns) # employment status (1 unemployed 2 employed)
        ls[k] = Ls[vE[1]] ## labor supply
        inc[k] = ls[k]*w*states[yy] # per capita
        id0 = 1+ (vE[N]-1)*na  # index of the initial distribution in D_ss
        if0 =  (vE[N])*na
        DD = 0 .*D_ss
        DD[id0:if0] .= D_ss[id0:if0]
        for j=1:N-1
                Mat_trans = 0 .*Mat                    
                id0 = 1+ (vE[N+1-j]-1)*na
                if0 =  (vE[N+1-j])*na
                id1 = 1+ (vE[N-j]-1)*na
                if1 =  (vE[N-j])*na
                Mat_trans[id1:if1,id0:if0] = Mat[id1:if1,id0:if0]                    
                DD = Mat_trans*DD                    
         end   
        
     Size_check[k] = sum(DD)
   #  CC[k]         = sum(CCv.*(Mat*DD))/Size_check[k] # fraction of CC agent in k
     CC[k]         = sum(resE.*DD)/Size_check[k] # fraction of CC agent in k   
     conso[k]      = sum(DD.*Cpol)
     at[k]         = sum(dGridl_t.*DD) # beginnning of period
     ae2[k]        = sum(dGridl_t.*(Mat*DD)) #at[k]*Aggs_ss.R + inc[k] - conso[k] (to check)
     ae[k]         = at[k]*Aggs_ss.R + inc[k]*Size_check[k] + Aggs_ss.TT*Size_check[k] - conso[k]   
     uc[k]         = sum(upc.*DD)
     Euc[k]        = sum(upc.*(Mat*DD)) # following utility    
     X[k]          = conso[k]/Size_check[k] - (ls[k]^(1+1/ϕ))/(χ*(1+1/ϕ))  
     Ul[k]         = Uvv(X[k]) 
    # xsiU[k]       = (sum(Ulevel.*DD)/Size_check[k])/Ul[k] 
    # xsiUc[k]      = (uc[k]/Size_check[k])/(X[k]^-γ) 
     resp[k]       = sum(DD.*resi)/ Size_check[k] 
 end
    
### transition Matrix for agents and wealth
#constructing the transition matrices on the projection
#Transp       = zeros(Float64,Nbin,Nbin); # general transition matrix AGENTS
    Transp       = spzeros(Nbin,Nbin); # general transition matrix AGENTS
for k  = 1:Nbin
    i  = Ntoi[k] #number of the history
    v  = convertBasis10E(i,Nidio,N)
    statei   = v[1]
    vf = v                     # to initialize
    vf[2:end] = v[1:end-1]     # translated vector
    for statej=1:Nidio
      # vf = v     
       if Trans[statej,statei]>0 # if possible continuation  
       vf[1] = statej 
       iff = convertBasisE10(vec(vf),Nidio,N)
                j = findall(x->x==iff,Ntoi) # index of the history
                if isempty(j) 
                else j=j[1]
                     Transp[j,k] = Trans[statej,statei] # from k to j per capita
                  #   Transu[j,k] = Transu[j,k]/uc[j]
                end               
       end
   end
end  

####################
#Defined CC bin
#################### 
# need a constrained CC    

Th = sort(CC,rev=true)[nc] # the item nc of CC in decreasing order
        
Id = sparse(I,Nbin,Nbin)
indcc=findall(x->x>=Th,CC)        # index of bin where fraction of CC is above Th
indnc=findall(x->x<Th,CC)        # index of bin where fraction of CC is above Th
    
#indcc = argmax(proj.Res)
PP = copy(Id)
for ii=1:length(indcc)
    PP[indcc[ii],indcc[ii]]= 0
end
PPc = Id - PP
   

Onev = ones(Float64,Nbin) 
Ucv = X.^-γ
Mat2 = PP*(Id - β*R*(Transp'))*diagm(Ucv) + PPc    
xsiUc = inv(Mat2)*PPc*Onev    
Res = ((xsiUc.*(X.^-γ)) -  β*R*(Transp')*(xsiUc.*(X.^-γ)))

#Onev = ones(Float64,Nbin) 
#SUcv = Size_check.*(X.^-γ)
#Mat2 = PP*(Id - β*R*(Transp'))*diagm(SUcv) + PPc    
#xsiUc = inv(Mat2)*PPc*Onev    
#Res = (Size_check.*(xsiUc.*(X.^-γ)) -  β*R*(Transp')*(Size_check.*xsiUc.*(X.^-γ)))


    
    
    
    
####################
#Defining types on the projection
####################     

ytypeb = zeros(Int64,Nbin)

    for k=1:Nbin
        i = Ntoi[k] #number of the history
        vE =  convertBasis10E(i, Nidio, N)
        ytypeb[k] = mod(vE[1] - 1 ,ns)+1 # productivity type between 1 and ns
        
end  

####################    
## computing the standard deviation of the transfer
####################

EΓ   = 0
EΓc   = 0
EΓ2 = 0
    
for i=1:Nbin
        Vi = zeros(Float64,size(Transp)[1])
        Vi[i]=1
        ind = findall(x->x>10^-6,Transp*Vi) # vector of possible successors of history i
        for j=ind
            EΓc = EΓc +Size_check[i]*Transp[j,i]*(at[j]/Size_check[j]-ae[i]/Size_check[i]) # should be 0
            EΓ = EΓ +Size_check[i]*Transp[j,i]*((at[j]/Size_check[j]-ae[i]/Size_check[i]) /(at[j]*Aggs_ss.R/Size_check[j] + inc[j] ) ) # divided by income pze capita
            EΓ2 = EΓ2 + Size_check[i]*Transp[j,i]*((at[j]/Size_check[j]-ae[i]/Size_check[i]) /(at[j]*Aggs_ss.R/Size_check[j] + inc[j] ) )^2
            
           #    EΓ = EΓ +Size_check[i]*Transp[j,i]*(at[j]/Size_check[j]-ae[i]/Size_check[i]) # divided by income pze capita
            #EΓ2 = EΓ2 + Size_check[i]*Transp[j,i]*(at[j]/Size_check[j]-ae[i]/Size_check[i] )^2
        end
end

    stdΓ = (EΓ2 - EΓ^2)^0.5
 
    
 
    Proj = Projection(
    Aggs_ss,
    N,        
    ns,        # number of states
    Nbin,         # number of bins
    R,
    w,
    A, #capital stock
    B,    
    states,
    Transv,    
    Size_check,  
    at,
    ae,  
    Ls, 
    conso, # conso (not per capita)
    Ucv,    
    Transp, 
    uc,#average marginal utility in the bin
    Ul,# utility in level of the bin (not ajusted)
    Res,    #xsiU,#level
    xsiUc,#marginal
    Ntoi,
    CC,
    indnc,
    indcc,    
    ytypeb)    
    
    return Proj,upc,resE, stdΓ, resi,ytype
end

